@@ -55,11 +55,13 @@ struct _PersonalIpAddressPrivate
 	GtkWidget *event;
     GtkWidget *contents;
 	
+	GtkWidget *ipImage;
 	GtkWidget *ipLabel;
 	GtkWidget *ipContent;
 	GtkWidget *interfaceContent;
 	
 	gboolean connectOnPress;
+	gboolean compactMode;
 	gboolean isPressed;
 	
 	ConIcConnection *connection;
@@ -91,8 +93,15 @@ void personal_ip_address_read_settings ( PersonalIpAddress *self )
             g_error_free (error);
             error = NULL;
         }
+		self->priv->compactMode = g_key_file_get_boolean (keyFile, "config", "compactMode", &error);
+		if (error) {
+			self->priv->compactMode = TRUE;
+			g_error_free (error);
+			error = NULL;
+		}
     } else {
         self->priv->connectOnPress = FALSE;
+		self->priv->compactMode = TRUE;
     }
 
     g_key_file_free (keyFile);
@@ -113,7 +122,8 @@ void personal_ip_address_write_settings (PersonalIpAddress *self)
 	
 	keyFile = g_key_file_new();
 		
-	g_key_file_set_integer (keyFile, "config", "connectOnPress", self->priv->connectOnPress);
+	g_key_file_set_boolean (keyFile, "config", "connectOnPress", self->priv->connectOnPress);
+	g_key_file_set_boolean (keyFile, "config", "compactMode", self->priv->compactMode);
 	
 	filename = g_strconcat (HOME_DIR, "/.personal_ip_address", NULL);
 	fileData = g_key_file_to_data (keyFile, &size, NULL);
@@ -163,17 +173,25 @@ void personal_ip_address_content_create (PersonalIpAddress *self)
 	gtk_container_set_border_width (GTK_CONTAINER (self->priv->contents), 0);
 	GtkSizeGroup *group = GTK_SIZE_GROUP (gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL));
 
+	GtkIconTheme *theme = gtk_icon_theme_get_default ();
+	if (gtk_icon_theme_has_icon (theme, "personal-ip-address")) {
+		self->priv->ipImage = gtk_image_new_from_icon_name ("personal-ip-address", GTK_ICON_SIZE_BUTTON);
+	} else {
+		self->priv->ipImage = gtk_image_new_from_file ("/usr/share/icons/hicolor/64x64/hildon/personal-ip-address.png");
+	}
 	self->priv->ipLabel = gtk_label_new ("IP");
 	GtkWidget *box = gtk_hbox_new (FALSE, 0);
 	self->priv->interfaceContent = gtk_label_new ("");
 	self->priv->ipContent = gtk_label_new ("(no IP found)");
+	gtk_box_pack_start (GTK_BOX (box), self->priv->ipImage, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (box), self->priv->ipLabel, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (box), self->priv->interfaceContent, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (box), self->priv->ipContent, TRUE, TRUE, 0);
 	hildon_helper_set_logical_color (self->priv->ipContent, GTK_RC_FG, GTK_STATE_NORMAL, "ActiveTextColor");
 	hildon_helper_set_logical_color (self->priv->interfaceContent, GTK_RC_FG, GTK_STATE_NORMAL, "ActiveTextColor");
-	gtk_misc_set_alignment (GTK_MISC (self->priv->ipLabel), 0, 0);
+	gtk_misc_set_alignment (GTK_MISC (self->priv->ipLabel), 0, 1);
 	gtk_misc_set_alignment (GTK_MISC (self->priv->ipContent), 1, 1);
+	gtk_misc_set_alignment (GTK_MISC (self->priv->interfaceContent), 0, 1);
 	gtk_misc_set_padding (GTK_MISC (self->priv->ipContent), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
 	gtk_misc_set_padding (GTK_MISC (self->priv->interfaceContent), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
 	gtk_misc_set_padding (GTK_MISC (self->priv->ipLabel), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
@@ -186,6 +204,13 @@ void personal_ip_address_content_create (PersonalIpAddress *self)
 	g_signal_connect (self->priv->contents, "leave-notify-event", G_CALLBACK (personal_ip_address_leave_event), self);
 	
     gtk_widget_show_all (self->priv->homeWidget);
+	if (self->priv->compactMode) {
+		gtk_widget_hide (self->priv->ipImage);
+		gtk_widget_show (self->priv->ipLabel);
+	} else {
+		gtk_widget_hide (self->priv->ipLabel);
+		gtk_widget_show (self->priv->ipImage);
+	}
 }
 
 gboolean personal_ip_address_update_content (PersonalIpAddress *self)
@@ -248,10 +273,28 @@ void personal_ip_address_settings (HDHomePluginItem *hitem, PersonalIpAddress *s
 	gtk_container_add (GTK_CONTAINER (content_area), connectOnPress);
 	hildon_check_button_set_active (HILDON_CHECK_BUTTON (connectOnPress), self->priv->connectOnPress);
 	
+	GtkWidget *compactMode = hildon_check_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
+	gtk_button_set_label (GTK_BUTTON (compactMode), "Use compact layout");
+	gtk_container_add (GTK_CONTAINER (content_area), compactMode);
+	hildon_check_button_set_active (HILDON_CHECK_BUTTON (compactMode), self->priv->compactMode);
+	
 	gtk_widget_show_all (dialog);
 	
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
 		self->priv->connectOnPress = hildon_check_button_get_active (HILDON_CHECK_BUTTON (connectOnPress));
+		self->priv->compactMode = hildon_check_button_get_active (HILDON_CHECK_BUTTON (compactMode));
+		
+		if (self->priv->compactMode) {
+			gtk_widget_hide (self->priv->ipImage);
+			gtk_widget_show (self->priv->ipLabel);
+			gtk_widget_set_size_request (GTK_WIDGET (self), 315, 40);
+			gtk_window_resize (GTK_WINDOW (self), 315, 40);
+		} else {
+			gtk_widget_hide (self->priv->ipLabel);
+			gtk_widget_show (self->priv->ipImage);
+			gtk_widget_set_size_request (GTK_WIDGET (self), 315, 56);
+			gtk_window_resize (GTK_WINDOW (self), 315, 56);
+		}
 		
 		personal_ip_address_write_settings (self);
 		personal_ip_address_update_content (self);
@@ -334,6 +377,11 @@ static gboolean personal_ip_address_expose_event (GtkWidget *widget, GdkEventExp
 	x = widget->allocation.x;
 	y = widget->allocation.y;
 
+	if (!self->priv->compactMode) {
+		height = 40;
+		y += 16;
+	}
+
 	cairo_move_to(cr, x + radius, y);
 	cairo_line_to(cr, x + width - radius, y);
 	cairo_curve_to(cr, x + width - radius, y, x + width, y, x + width,
@@ -394,7 +442,7 @@ static void personal_ip_address_init (PersonalIpAddress *self)
 	hd_home_plugin_item_set_settings (&self->parent, TRUE);
 	g_signal_connect (&self->parent, "show-settings", G_CALLBACK (personal_ip_address_settings), self);
 
-	gtk_window_set_default_size (GTK_WINDOW (self), 315, 40);
+	gtk_window_set_default_size (GTK_WINDOW (self), 315, 56);
 	
 	self->priv->homeWidget = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (self), self->priv->homeWidget);
@@ -404,11 +452,19 @@ static void personal_ip_address_init (PersonalIpAddress *self)
 	hints.min_width = 315;
 	hints.min_height = 40;
 	hints.max_width = 315;
-	hints.max_height = 40;
+	hints.max_height = 56;
 	gtk_window_set_geometry_hints (GTK_WINDOW (self), self->priv->homeWidget, &hints, GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE);
 	
 	personal_ip_address_read_settings (self);
 	
+	if (self->priv->compactMode) {
+		gtk_widget_set_size_request (GTK_WIDGET (self), 315, 40);
+		gtk_window_resize (GTK_WINDOW (self), 315, 40);
+	} else {
+		gtk_widget_set_size_request (GTK_WIDGET (self), 315, 56);
+		gtk_window_resize (GTK_WINDOW (self), 315, 56);
+	}
+	
 	personal_ip_address_content_create (self);
 	personal_ip_address_update_content (self);
 	
